"use client"

import { ImageVariant, type Color, type Image, type ImageColor } from "@/generated/prisma"
import { useGlobalSettings } from "@/hooks/useGlobalSettings"
import clsx from "clsx"
import { EyeOffIcon } from "lucide-react"
import NextImage from "next/image"
import Link from "next/link"
import { GlowingBorderWrapper } from "./GlowingBorderWrapper"
import { TagBadge } from "./TagBadge"

type ImageWithColors = Image & {
  colors?: (ImageColor & { color: Color })[],
  variants?: ImageVariant[]
}

type ImageCardProps = {
  image: ImageWithColors;
  gallerySlug?: string;
  albumSlug?: string;
  size?: "default" | "large";
  disableLink?: boolean;
}

export default function ImageCard({
  image,
  gallerySlug,
  albumSlug,
  size = "default",
  disableLink = false,
}: ImageCardProps) {
  const { showNSFW, animateGlow } = useGlobalSettings()
  const shouldBlur = image.nsfw && !showNSFW

  const isLarge = size === "large"
  const href = isLarge
    ? `/raw/${image.id}`
    : `/galleries/${gallerySlug}/${albumSlug}/${image.id}`

  const borderColors =
    image.colors?.map((c) => c.color?.hex).filter((hex): hex is string => Boolean(hex)) ?? []

  const thumbnailSrc = `/api/image/thumbnails/${image.fileKey}.webp`
  const resizedVariant = image.variants?.find((v) => v.type === "resized")
  const resizedSrc = resizedVariant ? `/api/image/${resizedVariant.s3Key}` : undefined

  const imageSrc = isLarge && resizedSrc ? resizedSrc : thumbnailSrc
  const width = isLarge && resizedVariant?.width ? resizedVariant.width : undefined
  const height = isLarge && resizedVariant?.height ? resizedVariant.height : undefined

  const imageElement = (
    <div className="group rounded-lg border overflow-hidden hover:shadow-lg transition-shadow bg-background relative">
      <div className="relative w-full bg-muted items-center justify-center"
        style={isLarge && width && height ? { width, height } : { aspectRatio: "4 / 3" }}
      >
        <NextImage
          src={imageSrc}
          alt={image.imageName}
          fill={!width || !height}
          width={width}
          height={height}
          className={clsx("object-cover transition duration-300", shouldBlur && "blur-md scale-105")}
        />
        {image.nsfw && (
          <div className="absolute top-1 left-1 z-10 flex gap-1">
            <TagBadge
              label="NSFW"
              variant="destructive"
              icon={<EyeOffIcon size={12} />}
              className="text-xs px-2 py-0.5 inline-flex items-center"
            />
          </div>
        )}
      </div>
      {!isLarge && (
        <div className="p-4">
          <h2 className="text-lg font-semibold truncate text-center">{image.imageName}</h2>
        </div>
      )}
    </div>
  )


  return disableLink ? (
    animateGlow && borderColors.length > 0 ? (
      <GlowingBorderWrapper colors={borderColors} size={size}>{imageElement}</GlowingBorderWrapper>
    ) : (
      imageElement
    )
  ) : (
    <Link href={href}>
      {animateGlow && borderColors.length > 0 ? (
        <GlowingBorderWrapper colors={borderColors} size={size}>{imageElement}</GlowingBorderWrapper>
      ) : (
        imageElement
      )}
    </Link>
  )
}

